/*
  HardwareSerial.h - Hardware serial library for Wiring
  Copyright (c) 2006 Nicholas Zambetti.  All right reserved.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

  Modified 28 September 2010 by Mark Sproul
*/

#ifndef HardwareSerial_h
#define HardwareSerial_h

#include <inttypes.h>
#include <string.h>

struct ring_buffer;

class HardwareSerial 
{
  private:
    ring_buffer *_rx_buffer;
    ring_buffer *_tx_buffer;
    volatile uint8_t *_ubrrh;
    volatile uint8_t *_ubrrl;
    volatile uint8_t *_ucsra;
    volatile uint8_t *_ucsrb;
    volatile uint8_t *_udr;
    uint8_t _rxen;
    uint8_t _txen;
    uint8_t _rxcie;
    uint8_t _udrie;
    uint8_t _u2x;
  public:
    HardwareSerial();/*ring_buffer *rx_buffer, ring_buffer *tx_buffer,
      volatile unsigned char *ubrrh, volatile unsigned char *ubrrl,
      volatile unsigned char *ucsra, volatile unsigned char *ucsrb,
      volatile unsigned char *udr,
      unsigned char rxen, unsigned char txen, unsigned char rxcie, unsigned char udrie, unsigned char u2x*/
    size_t printNumber(unsigned long, uint8_t);
    size_t printFloat(double, uint8_t);
    void begin(unsigned long);
    void end();
    int available();
    int peek(void);
    int read(void);
    void flush(void);
    virtual size_t write(uint8_t);
//    virtual size_t write(uint8_t) = 0;
    size_t write(const char *str) { return write((const uint8_t *)str, strlen(str)); }
    size_t write(const uint8_t *buffer, size_t size);

};

//#if defined(UBRRH) || defined(UBRR0H)
//  extern HardwareSerial Serial;
//#elif defined(USBCON)
//  #include "USBAPI.h"
//  extern HardwareSerial Serial_;  
//#endif
#if defined(UBRR1H)
  extern HardwareSerial Serial1;
#endif
#if defined(UBRR2H)
  extern HardwareSerial Serial2;
#endif
#if defined(UBRR3H)
  extern HardwareSerial Serial3;
#endif

extern void serialEventRun(void) __attribute__((weak));

#endif
